/* This file is part of the db4o object database http://www.db4o.com

Copyright (C) 2004 - 2009  Versant Corporation http://www.versant.com

db4o is free software; you can redistribute it and/or modify it under
the terms of version 3 of the GNU General Public License as published
by the Free Software Foundation.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see http://www.gnu.org/licenses/. */
using Db4objects.Db4o.Ext;
using Db4objects.Db4o.Internal;
using Db4objects.Db4o.Internal.CS;
using Db4objects.Db4o.Internal.CS.Messages;

namespace Db4objects.Db4o.Internal.CS.Messages
{
	public sealed class MCommit : Msg, IServerSideMessage
	{
		public bool ProcessAtServer()
		{
			try
			{
				CallbackObjectInfoCollections committedInfo = null;
				LocalTransaction serverTransaction = ServerTransaction();
				IServerMessageDispatcher dispatcher = ServerMessageDispatcher();
				lock (StreamLock())
				{
					serverTransaction.Commit(dispatcher);
					committedInfo = dispatcher.CommittedInfo();
				}
				Write(Msg.Ok);
				if (committedInfo != null)
				{
					AddCommittedInfoMsg(committedInfo, serverTransaction);
				}
			}
			catch (Db4oException e)
			{
				WriteException(e);
			}
			return true;
		}

		private void AddCommittedInfoMsg(CallbackObjectInfoCollections committedInfo, LocalTransaction
			 serverTransaction)
		{
			lock (StreamLock())
			{
				Msg.CommittedInfo.SetTransaction(serverTransaction);
				MCommittedInfo message = Msg.CommittedInfo.Encode(committedInfo);
				message.SetMessageDispatcher(ServerMessageDispatcher());
				ServerMessageDispatcher().Server().AddCommittedInfoMsg(message);
			}
		}
	}
}
